/*
 * libusb example program to list devices on the bus
 * Copyright © 2007 Daniel Drake <dsd@gentoo.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include <stdio.h>

#include "libusb.h"

#define NUM_CONTEXTS 3

static void print_devs(libusb_device **devs);
int default_context(void);
int multiple_contexts(void);

static void print_devs(libusb_device **devs)
{
	libusb_device *dev;
	int i = 0, j = 0;
	uint8_t path[8]; 

	while ((dev = devs[i++]) != NULL) {
		struct libusb_device_descriptor desc;
		int r = libusb_get_device_descriptor(dev, &desc);
		if (r < 0) {
			fprintf(stderr, "failed to get device descriptor");
			return;
		}

		printf("%04x:%04x (bus %d, device %d)",
			desc.idVendor, desc.idProduct,
			libusb_get_bus_number(dev), libusb_get_device_address(dev));

		r = libusb_get_port_numbers(dev, path, sizeof(path));
		if (r > 0) {
			printf(" path: %d", path[0]);
			for (j = 1; j < r; j++)
				printf(".%d", path[j]);
		}
		printf("\n");
	}
}

int default_context() {
	libusb_device **devs;
	int r;
	ssize_t cnt;

	printf("Custom\n");

	printf("Init:\n");
	r = libusb_init(NULL);
	printf("Done:\n");
	if (r < 0)
		return r;

	printf("Get device list:\n");
	cnt = libusb_get_device_list(NULL, &devs);
	if (cnt < 0)
		return (int) cnt;
	printf("Get device list: done\n");

	print_devs(devs);
	libusb_free_device_list(devs, 1);

	while(1);

	libusb_exit(NULL);

	return 0;
}



int multiple_contexts() {
	libusb_device **devs;
	int r, i;
	ssize_t cnt;
	libusb_context *contexts[NUM_CONTEXTS];

	printf("Using contexts: %d\n", NUM_CONTEXTS);

	for(i = 0; i < NUM_CONTEXTS; i++) {

		printf("\tInit context %d\n", i);
		r = libusb_init(&(contexts[i]));
		/*if (r < 0){ 
			printf("\tInit failed. Exiting\n");
			return r;
		}*/


	}


	printf("Getting dev lists\n");

	for(i = 0; i < NUM_CONTEXTS; i++) {
		printf("\tGetting dev list for context %d\n", i);
		cnt = libusb_get_device_list(contexts[i], &devs);
		if (cnt < 0){
			printf("\tGetting dev list for context failed. Exiting.\n");
			return (int) cnt;
		}

		print_devs(devs);

		printf("\tFreeing dev list for context %d\n", i);
		libusb_free_device_list(devs, 1);

		printf("\tFreeing context %d\n", i);
		libusb_exit(contexts[i]);
	}

	return 0;
}

int main(int argc, char *argv[])
{
	if(argc > 1){
		multiple_contexts();
	} else {
		default_context();
	}
	return 0;
}
